import * as momentImported from 'moment';
import { Component, Input, Output, EventEmitter, NgModule } from '@angular/core';

/**
 * @fileoverview added by tsickle
 * Generated from: lib/angular-formio-pdf/angular-formio-pdf.component.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const moment = momentImported;
class AngularFormioPdfComponent {
    constructor() {
        this.onRenderBase64 = new EventEmitter();
        this.onRenderBuffer = new EventEmitter();
        this.onRenderBlob = new EventEmitter();
        this.pdfconfig = {
            footer: (/**
             * @param {?} currentPage
             * @param {?} pageCount
             * @return {?}
             */
            function (currentPage, pageCount) {
                return [{ text: currentPage.toString() + ' of ' + pageCount, alignment: 'right', margin: [0, 40, 20, 15] }];
            }),
            pageSize: 'A4',
            pageOrientation: 'portrait',
            pageMargins: [40, 40, 40, 80],
            content: [],
            styles: {
                header: {
                    fontSize: 18,
                    bold: true,
                    margin: [0, 0, 0, 15],
                    alignment: 'center'
                },
                label: {
                    bold: true,
                    fontSize: 14,
                    lineHeight: 1.3
                },
                value: {
                    height: 30,
                    color: '#222'
                },
                table: {
                    margin: [0, 5, 0, 15]
                },
                tableHeader: {
                    bold: true,
                    fontSize: 13,
                    color: 'black',
                    alignment: 'center',
                    margin: [0, 5, 0, 5]
                },
                para: {
                    lineHeight: 1.5,
                    alignment: 'center',
                    margin: [0, 10, 0, 15]
                },
                columns: {
                    margin: [0, 15, 0, 15]
                },
                signature: {
                    width: 520
                }
            },
            defaultStyle: {
                columnGap: 20
            }
        };
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.pdf = Object.assign({}, this.pdfconfig, this.pdf);
        this.simplifiedDate = this.component.map((/**
         * @param {?} v
         * @return {?}
         */
        (v) => {
            return this.getSimplifiedLayout(v);
        }));
        this.simplifiedDate.map((/**
         * @param {?} v
         * @return {?}
         */
        (v) => {
            this.getSimplifiedPdfLayout(v);
        }));
        this.generatepdf();
    }
    /**
     * @return {?}
     */
    generatepdf() {
        /** @type {?} */
        const pdfDocGenerator = pdfMake.createPdf(this.pdfconfig);
        pdfDocGenerator.getDataUrl((/**
         * @param {?} dataUrl
         * @return {?}
         */
        (dataUrl) => {
            /** @type {?} */
            const targetElement = document.getElementById('angularFormIOPdf');
            /** @type {?} */
            const iframe = this.createiframe();
            iframe.src = dataUrl;
            targetElement.appendChild(iframe);
        }));
        pdfDocGenerator.getBase64((/**
         * @param {?} data
         * @return {?}
         */
        (data) => {
            // alert(data);
            this.onRenderBase64.emit(data);
        }));
        pdfDocGenerator.getBuffer((/**
         * @param {?} buffer
         * @return {?}
         */
        (buffer) => {
            this.onRenderBuffer.emit(buffer);
        }));
        pdfDocGenerator.getBlob((/**
         * @param {?} blob
         * @return {?}
         */
        (blob) => {
            this.onRenderBlob.emit(blob);
        }));
    }
    /**
     * @return {?}
     */
    createiframe() {
        /** @type {?} */
        const iframe = document.createElement('iframe');
        iframe.setAttribute('id', 'ifAssessmentView');
        iframe.setAttribute('name', 'ifAssessmentView');
        iframe.setAttribute('frameborder', '0');
        iframe.setAttribute('webkitallowfullscreen', '');
        iframe.setAttribute('mozallowfullscreen', '');
        iframe.setAttribute('allowfullscreen', '');
        iframe.setAttribute('style', 'width: -webkit-fill-available; height: -webkit-fill-available;');
        return iframe;
    }
    /**
     * @param {?} data
     * @return {?}
     */
    getSimplifiedLayout(data) {
        if (data.type === 'columns') {
            return this.getLayoutColumns(data);
        }
        else if (data.type === 'fieldset' || data.type === 'panel' || data.type === 'well') {
            if (data.type === 'panel') {
                return {
                    type: data.type,
                    value: this.submission[data.key] ? this.submission[data.key] : '',
                    key: data.key,
                    hidelabel: false,
                    label: data.title,
                    input: data.input,
                    component: this.getComponent(data.components)
                };
            }
            else {
                return {
                    type: data.type,
                    value: this.submission[data.key] ? this.submission[data.key] : '',
                    key: data.key,
                    hidelabel: data.hideLabel,
                    label: data.label,
                    input: data.input,
                    component: this.getComponent(data.components)
                };
            }
        }
        else if (data.type === 'table') {
            return {
                type: data.type,
                value: this.submission[data.key] ? this.submission[data.key] : '',
                key: data.key,
                hidelabel: data.hideLabel,
                label: data.label,
                input: data.input,
                component: data.rows.map((/**
                 * @param {?} k
                 * @return {?}
                 */
                (k) => {
                    return k.map((/**
                     * @param {?} c
                     * @return {?}
                     */
                    (c) => {
                        return this.getComponent(c.components);
                    }));
                }))
            };
        }
        else if (data.input) {
            /** @type {?} */
            let out = {
                type: data.type,
                value: this.submission[data.key] ? this.submission[data.key] : '',
                key: data.key,
                hidelabel: data.hideLabel,
                label: data.label,
                input: data.input,
                component: data.components && data.components.length ? this.getComponent(data.components) : undefined
            };
            if (data.type === 'datetime') {
                out.value = this.submission[data.key] ? moment(this.submission[data.key]).format('DD-MM-YYYY') : '';
            }
            else if (data.key === 'submit') {
                out.value = '';
                out.hidelabel = true;
            }
            return out;
        }
    }
    /**
     * @param {?} v
     * @return {?}
     */
    getLayoutColumns(v) {
        /** @type {?} */
        const obj = {
            type: v.type,
            value: this.submission[v.key] ? this.submission[v.key] : '',
            key: v.key,
            label: v.label,
            component: v.columns.map((/**
             * @param {?} x
             * @return {?}
             */
            (x) => {
                if (x.type === 'column') {
                    return {
                        type: x.type,
                        value: this.submission[x.key] ? this.submission[x.key] : '',
                        key: x.key,
                        hidelabel: x.hideLabel,
                        label: x.label,
                        input: false,
                        component: x.components && x.components.length ? this.getComponent(x.components) : undefined
                    };
                }
                else {
                    return v.type;
                }
            }))
        };
        obj.component = [].concat.apply([], obj.component);
        return obj;
    }
    /**
     * @param {?} data
     * @return {?}
     */
    getComponent(data) {
        return data.map((/**
         * @param {?} k
         * @return {?}
         */
        (k) => {
            if (k.input && k.type !== 'button' && k.key !== 'submit') {
                if (k.type === 'signature' && this.submission[k.key] !== '') {
                    return {
                        type: k.type,
                        value: this.submission[k.key] ? this.submission[k.key] : '',
                        key: k.key,
                        image: true,
                        hidelabel: true,
                        label: k.label,
                        input: k.input,
                        component: k.components && k.components.length ? this.getComponent(k.components) : undefined
                    };
                }
                else {
                    /** @type {?} */
                    let out = {
                        type: k.type,
                        value: this.submission[k.key] ? this.submission[k.key] : '',
                        key: k.key,
                        image: false,
                        hidelabel: k.hideLabel,
                        label: k.label,
                        input: k.input,
                        component: k.components && k.components.length ? this.getComponent(k.components) : undefined
                    };
                    if (k.type === 'checkbox' || k.type === 'radio') {
                        out.hidelabel = true;
                    }
                    if (k.type === 'datetime') {
                        out.value = this.submission[k.key] ? moment(this.submission[k.key]).format('DD-MM-YYYY') : '';
                    }
                    return out;
                }
            }
            else if (k.type === 'htmlelement') {
                if (k.content.includes('<img src=')) {
                    return {
                        type: k.type,
                        value: k.content.replaceAll('<img src="', '').replaceAll('" alt="">', '').replaceAll('<br>', ''),
                        key: k.key,
                        hidelabel: true,
                        image: true,
                        label: k.label,
                        input: k.input,
                        component: k.components && k.components.length ? this.getComponent(k.components) : undefined
                    };
                }
                else {
                    return {
                        type: k.type,
                        value: k.content.replaceAll('<center>', '').replaceAll('</center>', '').replaceAll('<input>', '').replaceAll('</input>', '')
                            .replaceAll('<b>', '').replaceAll('</b>', '').replaceAll('<p>', '').replaceAll('</p>', '')
                            .replaceAll('<br>', ''),
                        key: k.key,
                        image: false,
                        hidelabel: true,
                        label: k.label,
                        style: k.style,
                        margin: k.margin,
                        input: k.input,
                        component: k.components && k.components.length ? this.getComponent(k.components) : undefined
                    };
                }
            }
            else {
                return !k.input ? this.getSimplifiedLayout(k) : k.type;
            }
        }));
    }
    /**
     * @param {?} data
     * @return {?}
     */
    getSimplifiedPdfLayout(data) {
        if (data.type === 'columns') {
            if (data.component && data.component.length) {
                this.pdfconfig.content.push(this.getPdfLayoutColumns(data.component));
            }
        }
        else if (data.type === 'fieldset' || data.type === 'panel' || data.type === 'well') {
            if (data.type === 'panel') {
                if (!data.hidelabel) {
                    this.pdfconfig.content.push({
                        text: data.label,
                        style: 'header'
                    });
                }
                if (data.component && data.component.length) {
                    /** @type {?} */
                    let component = this.getPdfComponent(data.component, 'panel');
                    component = [].concat.apply([], component);
                    this.pdfconfig.content.push(component);
                }
            }
            else {
                this.getPdfInput(data);
            }
        }
        else if (data.type === 'table') {
            /** @type {?} */
            const tabel = {
                style: 'table',
                table: {
                    widths: data.component[0].map((/**
                     * @param {?} x
                     * @return {?}
                     */
                    (x) => '*')),
                    body: data.component.map((/**
                     * @param {?} k
                     * @param {?} ind
                     * @return {?}
                     */
                    (k, ind) => {
                        /** @type {?} */
                        let rows = k.map((/**
                         * @param {?} c
                         * @return {?}
                         */
                        (c) => {
                            /** @type {?} */
                            let columns;
                            if (ind === 0) {
                                columns = c.length ? this.getPdfComponent(c, 'table', true) : '';
                            }
                            else {
                                columns = c.length ? this.getPdfComponent(c, 'table') : '';
                            }
                            columns = typeof columns !== 'string' ? [].concat.apply([], columns) : '';
                            return columns;
                        }));
                        return rows;
                    }))
                }
            };
            this.pdfconfig.content.push(tabel);
        }
        else if (data.input) {
            this.pdfconfig.content.push(this.getPdfInput(data));
        }
    }
    /**
     * @param {?} v
     * @return {?}
     */
    getPdfLayoutColumns(v) {
        /** @type {?} */
        let col = v.map((/**
         * @param {?} x
         * @return {?}
         */
        (x) => {
            /** @type {?} */
            let component = x.component ? this.getPdfComponent(x.component, 'column') : '';
            component = typeof component !== 'string' ? [].concat.apply([], component) : '';
            return component;
        }));
        /** @type {?} */
        let updatedcol = col.map((/**
         * @param {?} v
         * @return {?}
         */
        (v) => {
            return {
                text: v === '' ? [] : v
            };
        }));
        updatedcol = [].concat.apply([], updatedcol);
        return {
            columns: updatedcol,
            style: 'columns'
        };
    }
    /**
     * @param {?} data
     * @param {?=} type
     * @param {?=} tablehead
     * @return {?}
     */
    getPdfComponent(data, type, tablehead) {
        return data.map((/**
         * @param {?} k
         * @return {?}
         */
        (k) => {
            if (k.input) {
                /** @type {?} */
                let arr = [];
                if (!k.hidelabel) {
                    arr.push({
                        text: `${k.label} \n`,
                        style: 'label'
                    });
                }
                if (k.image && k.value !== '') {
                    arr.push({
                        image: k.value,
                        style: 'signature'
                    });
                }
                else {
                    arr.push({
                        text: `${k.value}`,
                        style: 'value'
                    });
                }
                if (k.component && k.component.length) {
                    arr.push(this.getPdfComponent(k.component, 'inner'));
                }
                if (type === 'inner' || type === 'table' || type === 'column') {
                    return arr;
                }
                else {
                    this.pdfconfig.content.push(arr);
                }
            }
            else if (k.type === 'htmlelement') {
                /** @type {?} */
                let arr = [];
                if (k.image && k.value !== '') {
                    arr.push({
                        image: k.value
                    });
                }
                else {
                    if (tablehead) {
                        arr.push({
                            text: k.value,
                            style: 'tableHeader'
                        });
                    }
                    else {
                        /** @type {?} */
                        const styles = k.style ? k.style : {};
                        /** @type {?} */
                        let textWithStyles = Object.assign({ text: k.value, style: 'para' }, styles);
                        arr.push(textWithStyles);
                    }
                }
                if (k.component && k.component.length) {
                    arr.push(this.getPdfComponent(k.component, 'inner'));
                }
                if (type === 'table') {
                    return arr;
                }
                else {
                    this.pdfconfig.content = [...this.pdfconfig.content, ...arr];
                }
            }
            else {
                if (!k.input) {
                    this.getSimplifiedPdfLayout(k);
                }
            }
        }));
    }
    /**
     * @param {?} v
     * @return {?}
     */
    getPdfInput(v) {
        if (!v.hidelabel) {
            this.pdfconfig.content.push({
                text: `${v.label} \n`,
                style: 'label'
            });
        }
        this.pdfconfig.content.push({
            text: `${v.value}`,
            style: 'value'
        });
        if (v.component && v.component.length) {
            /** @type {?} */
            let arr = [];
            arr.push(this.getPdfComponent(v.component));
            arr = [].concat.apply([], arr);
            this.pdfconfig.content.push(arr);
        }
    }
}
AngularFormioPdfComponent.decorators = [
    { type: Component, args: [{
                selector: 'angular-formio-pdf',
                template: "<div id=\"angularFormIOPdf\"></div>\n",
                styles: [""]
            }] }
];
/** @nocollapse */
AngularFormioPdfComponent.ctorParameters = () => [];
AngularFormioPdfComponent.propDecorators = {
    component: [{ type: Input }],
    submission: [{ type: Input }],
    pdf: [{ type: Input }],
    onRenderBase64: [{ type: Output }],
    onRenderBuffer: [{ type: Output }],
    onRenderBlob: [{ type: Output }]
};

/**
 * @fileoverview added by tsickle
 * Generated from: lib/angular-formio-pdf.module.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AngularFormioPdfModule {
}
AngularFormioPdfModule.decorators = [
    { type: NgModule, args: [{
                declarations: [AngularFormioPdfComponent],
                imports: [],
                exports: [AngularFormioPdfComponent]
            },] }
];

/**
 * @fileoverview added by tsickle
 * Generated from: public-api.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * Generated from: angular-formio-pdf.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { AngularFormioPdfComponent, AngularFormioPdfModule };

//# sourceMappingURL=angular-formio-pdf.js.map